/*
 background.cpp - code for getting the desktop background
 Written by John Yanarella <yanarejm@muohio.edu>

 This code is derived from code from Open Tracker (www.opentracker.org):
 
	Open Tracker License
	
	Terms and Conditions
	
	Copyright (c) 1991-2000, Be Incorporated. All rights reserved.
	
	Permission is hereby granted, free of charge, to any person obtaining a copy of
	this software and associated documentation files (the "Software"), to deal in
	the Software without restriction, including without limitation the rights to
	use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
	of the Software, and to permit persons to whom the Software is furnished to do
	so, subject to the following conditions:
	
	The above copyright notice and this permission notice applies to all licensees
	and shall be included in all copies or substantial portions of the Software.
	
	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF TITLE, MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
	BE INCORPORATED BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
	AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF, OR IN CONNECTION
	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
	
	Except as contained in this notice, the name of Be Incorporated shall not be
	used in advertising or otherwise to promote the sale, use or other dealings in
	this Software without prior written authorization from Be Incorporated.
	
	Tracker(TM), Be(R), BeOS(R), and BeIA(TM) are trademarks or registered trademarks
	of Be Incorporated in the United States and other countries. Other brand product
	names are registered trademarks or trademarks of their respective holders.
	All rights reserved.
*/

#include <Message.h>
#include <TranslationKit.h>
#include <View.h>
#include <Point.h>
#include <Bitmap.h>

#include <FindDirectory.h>

#include <Path.h>
#include <Node.h>
#include <fs_attr.h>

#include "background.h"

const char *kBackgroundImageInfo 			= "be:bgndimginfo";
const char *kBackgroundImageInfoOffset 		= "be:bgndimginfooffset";
const char *kBackgroundImageInfoMode 		= "be:bgndimginfomode";
const char *kBackgroundImageInfoPath 		= "be:bgndimginfopath";

void SetBackgroundImage(BView* view, BBitmap* bitmap, Mode &mode, BPoint &offset)
{
	if (view && bitmap)
	{
		BRect viewBounds(view->Bounds());
		BRect bitmapBounds(bitmap->Bounds());
		BRect destinationBitmapBounds(bitmapBounds);
	
		uint32 tile = 0;
		uint32 followFlags = B_FOLLOW_TOP | B_FOLLOW_LEFT;
	
		// figure out the display mode and the destination bounds for the bitmap
		switch (mode) {
			case kCentered:
				destinationBitmapBounds.OffsetBy(
					(viewBounds.Width() - bitmapBounds.Width()) / 2,
					(viewBounds.Height() - bitmapBounds.Height()) / 2);
				break;
			case kScaledToFit:
				destinationBitmapBounds = viewBounds;
				followFlags = B_FOLLOW_ALL;
				break;
			case kAtOffset:
				destinationBitmapBounds.OffsetTo(offset);
				break;
			case kTiled:
				destinationBitmapBounds.OffsetBy(
					(viewBounds.Width() - bitmapBounds.Width()) / 2,
					(viewBounds.Height() - bitmapBounds.Height()) / 2);
				tile = B_TILE_BITMAP;
				break;
		}
		
		// switch to the bitmap and force a redraw
		view->SetViewBitmap(bitmap, bitmapBounds, destinationBitmapBounds, followFlags, tile);
		view->Invalidate();	
	}
}

BBitmap* GetCurrentBackgroundImage(Mode &mode, BPoint &offset)
{
	BPath desktopPath;
	find_directory(B_DESKTOP_DIRECTORY, &desktopPath);
	BNode node(desktopPath.Path());

	attr_info info;
	if (node.GetAttrInfo(kBackgroundImageInfo, &info) != B_OK)
		return NULL;
	
	BMessage container;
	char *buffer = new char [info.size];

	status_t error = node.ReadAttr(kBackgroundImageInfo, info.type, 0, buffer, (size_t)info.size);
	if (error == info.size)
		error = container.Unflatten(buffer);
		
	delete [] buffer;
	
	if (error != B_OK)
		return NULL;
	
	// set index to current workspace
	int32 index = current_workspace();
	
	mode = kTiled;
	offset.x = offset.y = 0.0;

	const char *path;
	if (container.FindString(kBackgroundImageInfoPath, index, &path) != B_OK)
		return NULL;
	
	container.FindInt32(kBackgroundImageInfoMode, index, (int32 *)&mode);
	if (mode == kAtOffset)
		container.FindPoint(kBackgroundImageInfoOffset, index, &offset);
	
	BBitmap *bitmap = BTranslationUtils::GetBitmapFile(path);

	return bitmap;
}